;; Maintain the members participating in each forum.
;; Copyright (C) 2001  Neal H Walfield  <neal@cs.uml.edu>
;;
;; This file is part of the Deride
;;
;; Deride is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;; 
;; Deride is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with the Deride; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

(define-module (forum users))

;; An alist forum names pointing to a list of strings of users.  Note,
;; forum names and user names are case-insensitive.
(define-public forum-list '())

;; USER has joined FORUM.  If USER is not already on the list, add him
;; and return #t, otherwise, return #f.
(define-public (forum-user-joined forum user)
  (set! forum (string-upcase forum))
  (let ((user-list (assoc forum forum-list)))
    (if user-list
	;; The forum exists.
	(begin
	  (if (not (member user user-list))
	      ;; But the user has not yet been added.
	      (set! forum-list
		    (assoc-set! forum-list forum (cons user (cdr user-list)))))
	  #t)
	;; This is first user in the forum.
	(begin
	  (set! forum-list (acons forum (list user) forum-list))
	  #f))))

;; USER has parted FORUM.  If he really was a member of the forum,
;; remove him from the user list and return #t, otherwise, return #f.
(define-public (forum-user-parted forum user)
  (set! forum (string-upcase forum))
  (let ((user-list (assoc forum forum-list)))
    (if user-list
	(begin (set! user-list (delete user (cdr user-list)))
	       (assoc-set! forum-list forum user-list)
	       #t)
	#f)))

;; USER has quit, i.e. parted all forums.  Remove him from all the
;; forums and return the list of forums that he was on.
(define-public (forum-user-quit user)
  (letrec ((chanlist '())
	   (loop (lambda (list)
		   (if (forum-user-parted (caar list) user)
		       (set! chanlist (cons (caar list) chanlist)))
		   (if (not (eqv? (cdr list) '()))
		       (loop (cdr list))))))
    (loop forum-list)
    chanlist))

;; USER changed their nick to NEWNAME.  Update all forums and return
;; the list of the forums that he is on.
(define-public (forum-user-schizophrenia user newname)
  (letrec ((chanlist '())
	   (loop (lambda (list)
		   (if (member user (cdar list))
		       (begin
 			 (let* ((forum (caar list))
				(user-list (cons newname
						 (delete user
							 (cdar list)))))
			   (assoc-set! forum-list forum user-list)
			   (set! chanlist (cons forum chanlist)))))
		   (if (not (eqv? (cdr list) '()))
		       (loop (cdr list))))))
    (loop forum-list)
    chanlist))

;; Return a list of strings containing the users in FORUM.
(define-public (forum-user-list forum)
  (let ((user-list
	 (assoc (string-upcase forum) forum-list)))
    (if user-list
	(cdr user-list)
	'())))

;; Return true if USER is present in FORUM, otherwise true.
(define-public (forum-user-is-present forum user)
  (if (member user (forum-user-list forum))
      #t
      #f))